/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.0
 * November 1998
 * ===========================
 *
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library 
 * Copyright (C) 1998, Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */
#ifndef _vdkb_cregex_h_
#define _vdkb_cregex_h_
#include <regex.h>
#include <string>
#include <vector>
#include <string.h>
using namespace std;
class  compiled_regex
{
protected:
    regex_t regex;
    int errnum;
public:
    compiled_regex( const char *pattern, int cflags=0)
    {
        errnum = regcomp( &regex, pattern, cflags);
    }
    virtual ~compiled_regex()
    {
        if( errnum == 0) regfree( &regex);
    }
}; 

class reported_regex : public compiled_regex
{
public:
    reported_regex( const char *pat, int cflags):compiled_regex( pat,cflags){};
    bool verify_regex( ostream& str) const
    {
        char buff[160];
        if(errnum != 0)
        {
            regerror( errnum, &regex, buff, 160);
            str<<buff<<endl;
            return false;
        }
        return true;
    }
};

class executed_regex : public reported_regex
{
    int nsubs;
    vector<string> subs;
public:
    executed_regex( const char *pat, int cflags, int ns)
      : reported_regex( pat, cflags)
    {
        nsubs=ns;
    }
    vector<string>::const_iterator begin() const { return subs.begin();}
    vector<string>::const_iterator end() const { return subs.end();}
    int regexec( const char *match, int flags=0)
    {
        regmatch_t * matches = NULL;
        if(nsubs > 0)
            matches = new regmatch_t[nsubs+1];
        subs.clear();
        int res= ::regexec(&regex,match,nsubs==0 ? 0 : nsubs+1,matches,flags);
        if(res == 0 && matches != NULL)
        {
            string tmpstr;
            int i;
            for(i=1;i<=nsubs;i++)
            {
                tmpstr="";
                if(matches[i].rm_so >= 0 && matches[i].rm_eo <= strlen(match))
                {
                    copy_n(match+matches[i].rm_so,
                           matches[i].rm_eo-matches[i].rm_so,
                           insert_iterator<string>(tmpstr,tmpstr.begin()));
                }
                subs.push_back(tmpstr);
            }
        }
        if(matches != NULL) delete matches;
        return res;
    }
};

#endif // _vdkb_cregex_h_

