/*
 * ===========================
 * VDK Builder
 * Version 2.0
 * March 2001
 * ===========================
 *
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library
 * Copyright (C) 1998, Mario Motta
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */
#if HAVE_CONFIG_H
#include <config.h>
#endif

#if !HAVE_GNOME
  #if ENABLE_NLS
    #include <libintl.h>
#define _(str) \
    ( g_utf8_validate(gettext(str),-1,NULL) ? \
    gettext(str) : \
    g_locale_to_utf8(gettext(str),-1,NULL,NULL,NULL) )
#define N_(str) str
  #else
    #define _(str) str
    #define N_(str) str 
  #endif 
#else
 #include <gnome.h>
#endif

#include <vdkb2/vdkb_FWizard.h>
#include "pixmaps/leftarrow.xpm"
#include "pixmaps/rightarrow.xpm"
#include "pixmaps/finish.xpm"
#include "pixmaps/stop.xpm"
#include "pixmaps/gnome-search.xpm"

/*
defining signal and events
dynamic tables
*/
DEFINE_SIGNAL_LIST(Vdkb_FWizardForm,VDKForm);
DEFINE_EVENT_LIST(Vdkb_FWizardForm,VDKForm);
/*
defining signal static table
*/
DEFINE_SIGNAL_MAP(Vdkb_FWizardForm,VDKForm)
  ON_SIGNAL(varlist,select_row_signal,OnvarlistSelectRow),
  ON_SIGNAL(delvar,clicked_signal,OndelvarClick),
  ON_SIGNAL(vartype,changed_signal,OnvartypeChanged),
  ON_SIGNAL(addvar,clicked_signal,OnaddvarClick),
  ON_SIGNAL(prev,clicked_signal,OnprevClick),
  ON_SIGNAL(next,clicked_signal,OnnextClick),
  ON_SIGNAL(finish,clicked_signal,OnfinishClick),
  ON_SIGNAL(cancel,clicked_signal,OncancelClick),
  ON_SIGNAL(emptydef,toggled_signal,OnemptydefToggled),
  ON_SIGNAL(deffilebtn,clicked_signal,OndeffilebtnClick)
//  ON_SIGNAL(typecombo,realize_signal,OntypecomboRealize)
END_SIGNAL_MAP

//define static display type and initial form position
GtkWindowType Vdkb_FWizardForm::DisplayType = GTK_WINDOW_TOPLEVEL;
GtkWindowPosition Vdkb_FWizardForm::InitialPosition = GTK_WIN_POS_CENTER;
/*
 form constructor
*/
Vdkb_FWizardForm::Vdkb_FWizardForm( VDKForm* owner, char* title, SFuncWizard* nfunc)
  : VDKForm(owner,title)
{
  func = nfunc;
}

/*
 form destructor
*/
Vdkb_FWizardForm::~Vdkb_FWizardForm()
{
}

/*
 form setup
*/
void
Vdkb_FWizardForm::Setup()
{
  GUISetup();

  // store frames addresses
  steps[0] = frame0;
  steps[1] = frame1;
  steps[1]->Visible = false;
  activeStep = 0;

}

/*
 form gui setup
*/
void
Vdkb_FWizardForm::GUISetup()
{
  SetSize(460,270);
  Title = "Testing only";
  vbox = new VDKBox(this,v_box);
  Add(vbox,0,1,1,0);
  frame0 = new VDKFrame(this,"Step 1 - Function declaration",v_box,shadow_etched_in);
  frame0->Align = c_justify;
  frame0->BorderWidth(5);
  vbox->Add(frame0,0,1,1,0);
  vbox0 = new VDKBox(this,v_box);
  frame0->Add(vbox0,l_justify,1,1,0);
  fixed0 = new VDKFixed(this);
  vbox0->Add(fixed0,0,1,1,0);
  fixed0->BorderWidth(2);
  fixed0->Add( new VDKLabel(this,"// member function access rights"),96,12,1,0);
  rettype = new VDKEntry(this,0);
  rettype->SetSize(110,20);
  rettype->Editable = true;
  fixed0->Add(rettype,8,32,1,0);
  fixed0->Add( new VDKLabel(this,"// returned value type"),128,36,1,0);
  fname = new VDKEntry(this,0);
  fname->SetSize(160,20);
  fname->Editable = true;
  fixed0->Add(fname,8,56,1,0);
  fixed0->Add( new VDKLabel(this,"// function name"),176,60,1,0);
  fixed0->Add( new VDKLabel(this,"( // function arguments"),8,80,1,0);
  char* varlistTitles[] = { 
    "Type",
    "Name" };
  varlist = new VDKCustomList(this,2,varlistTitles,(GtkSelectionMode) 0);
  varlist->SetSize(200,96);
  varlist->AutoResize = true;
  varlist->BorderShadow = (GtkShadowType) shadow_in;
  fixed0->Add(varlist,8,96,1,0);
  fixed0->Add( new VDKLabel(this,"); // end of function declaration"),8,192,1,0);
  addvar = new VDKCustomButton(this,(const char**)leftarrow_xpm,(char*) NULL,16,(GtkPositionType) 0);
  addvar->Enabled = false;
  addvar->SetSize(40,30);
  fixed0->Add(addvar,216,120,1,0);
  delvar = new VDKCustomButton(this,(const char**)stop_xpm,(char*) NULL,16,(GtkPositionType) 1);
  delvar->Enabled = false;
  delvar->SetSize(40,30);
  fixed0->Add(delvar,216,160,1,0);
  vartype = new VDKEntry(this,0);
  vartype->Editable = true;
  fixed0->Add(vartype,272,128,1,0);
  varname = new VDKEntry(this,0);
  varname->Editable = true;
  fixed0->Add(varname,272,168,1,0);
  fixed0->Add( new VDKLabel(this,"// argument type"),272,112,1,0);
  fixed0->Add( new VDKLabel(this,"// argument name"),272,152,1,0);
  typecombo = new VDKCombo(this);
  typecombo->SetSize(80,20);
  typecombo->Editable = false;
  fixed0->Add(typecombo,8,8,1,0);
  frame1 = new VDKFrame(this,"Step 2 - Function definition",v_box,shadow_etched_in);
  frame1->Align = c_justify;
  frame1->BorderWidth(5);
  vbox->Add(frame1,0,1,1,0);
  vbox1 = new VDKBox(this,v_box);
  frame1->Add(vbox1,l_justify,1,1,0);
  fixed1 = new VDKFixed(this);
  fixed1->BorderWidth(2);
  vbox1->Add(fixed1,0,1,1,0);
  functext = new VDKTextView(this);
  functext->SetSize(416,80);
  functext->Editable = false;
  fixed1->Add(functext,8,24,1,0);
  fixed1->Add( new VDKLabel(this,"Definition of created function:"),8,8,1,0);
  fixed1->Add( new VDKLabel(this,"will be:"),8,112,1,0);
  emptydef = new VDKCheckButton(this,"{ }");
  emptydef->SetSize(32,19);
  fixed1->Add(emptydef,8,128,1,0);
  fixed1->Add( new VDKLabel(this,"or edited in following file:"),8,152,1,0);
  deffilebtn = new VDKCustomButton(this,(const char*)gnome_search_xpm,"...",16,(GtkPositionType) 1);
  deffilebtn->SetSize(416,20);
  deffilebtn->CaptionWrap = true;
  deffilebtn->Relief = (GtkReliefStyle) 2;
  fixed1->Add(deffilebtn,8,168,1,0);
  vbox->Add( new VDKSeparator(this,h_separator),0,0,0,0);
  hbox = new VDKBox(this,h_box);
  vbox->Add(hbox,0,0,0,0);
  prev = new VDKCustomButton(this,(const char**)leftarrow_xpm,"_Previous step",16,(GtkPositionType) 1);
  prev->Relief = (GtkReliefStyle) 2;
  hbox->Add(prev,0,1,1,2);
  next = new VDKCustomButton(this,(const char**)rightarrow_xpm,"_Next step",16,(GtkPositionType) 1);
  next->Relief = (GtkReliefStyle) 2;
  hbox->Add(next,0,1,1,2);
  finish = new VDKCustomButton(this,(const char**)finish_xpm,"_Finish",16,(GtkPositionType) 1);
  finish->Relief = (GtkReliefStyle) 2;
  hbox->Add(finish,0,1,1,2);
  cancel = new VDKCustomButton(this,(const char**)stop_xpm,"_Cancel",16,(GtkPositionType) 1);
  cancel->Relief = (GtkReliefStyle) 2;
  hbox->Add(cancel,0,1,1,2);
  vbox->Add( new VDKSeparator(this,h_separator),0,0,0,0);
}

//signal response method
static char* acctype[] = { "public:", "protected:", "private:", NULL };

void 
Vdkb_FWizardForm::OnShow(VDKForm* sender) 
{
  StringList sl = typecombo->PopdownStrings;
  for ( int i = 0; acctype[i]; i++)
    sl.add ( acctype[i]);
  typecombo->PopdownStrings = sl;

  char buff[80];
  sprintf (buff, "New Function Wizard for class %s", func->parent);
  Title = buff;

  deffilebtn->Caption = func->decfile;

  prev->Enabled = false;
  finish->Enabled = false;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OndeffilebtnClick(VDKObject* sender) 
{
  char selection[512];
  *selection = '\0';
  VDKFileSel* child = new VDKFileSel (this, selection, 
                                      _("Choosing definition file"));
  child->ShowModal () ;
  if (*selection)
    {
      deffilebtn->Caption = selection;
      func->deffile = (char*)selection;
    }
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnemptydefToggled(VDKObject* sender) 
{
  deffilebtn->Enabled = ! emptydef->Checked;
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OncancelClick(VDKObject* sender) 
{
  func->isValid = false;
  Close();
  return true;
}

//signal response method
bool
Vdkb_FWizardForm::CanClose()
{
  return func->isValid;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnfinishClick(VDKObject* sender) 
{
  if (emptydef->Checked)
    func->deffile = (char*)NULL;
  
  func->isValid = true;
  Close ();
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnnextClick(VDKObject* sender) 
{
  static char text[512];
  if ( (activeStep ==  0) & SetFunc (text))
    {
      steps[activeStep]->Visible = false;
      if (activeStep < FMAX_STEP)
	activeStep++;
      prev->Enabled = activeStep > 0;
      next->Enabled = activeStep < FMAX_STEP;
      finish->Enabled = activeStep ==  FMAX_STEP;
      functext->Clear ();
      functext->TextInsert (text);
      steps[activeStep]->Visible = true;
    }
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnprevClick(VDKObject* sender) 
{
  steps[activeStep]->Visible = false;  
  if (activeStep > 0)
    activeStep--;
  prev->Enabled = activeStep > 0;
  next->Enabled =activeStep < FMAX_STEP;
  finish->Enabled = false;
  steps[activeStep]->Visible = true;    
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnaddvarClick(VDKObject* sender) 
{
  char* row[] = { vartype->Text, varname->Text};
  varlist->AddRow (row);
  vartype->Text = "";
  varname->Text = "";
  addvar->Enabled = false;
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnvartypeChanged(VDKObject* sender) 
{
  addvar->Enabled = true;
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OndelvarClick(VDKObject* sender) 
{
  varlist->RemoveRow ( varlist->Selected.Row ());
  delvar->Enabled = false;
  return true;
}
 
//signal response method
bool 
Vdkb_FWizardForm::OnvarlistSelectRow(VDKObject* sender) 
{
  delvar->Enabled = true;
  return true;
}
 
// set function parameters and check if it is valid
bool
Vdkb_FWizardForm::SetFunc (char* buff)
{
  func->access = typecombo->Selected;
  if ( func->access == -1)
    {
      Application ()->MessageBox (_("New Function Wizard"), 
				  _("You must select functions access rights!"), 
				  MB_OK | MB_ICONINFORMATION, 
				  NULL,  NULL,  5000);
      return false;
    }
  
  // set function return type
  func->rtype = rettype->Text;
  if ( *func->rtype == '\0')
    {
      Application ()->MessageBox (_("New Function Wizard"), 
				  _("You must enter functions return value type!"), 
				  MB_OK | MB_ICONINFORMATION, 
				  NULL,  NULL,  5000);
      return false;
    }
  
  //  set function name
  func->fname = fname->Text;
  if ( *func->fname == '\0')
    {
      Application ()->MessageBox (_("New Function Wizard"), 
				  _("You must enter functions name!"), 
				  MB_OK | MB_ICONINFORMATION, 
				  NULL,  NULL,  5000);
      return false;
    }
  
  // set function arguments if any,  but first clear old ones
  for ( int i = 0; i < FMAX_ARGS; i++)
    func->vtype[i] = func->vname[i] = '\0';

  if (varlist->Size ())
    {
      for ( int i = 0; i < varlist->Size ();i++)
	{
	  func->vtype[i] = (char*)varlist->Tuples[i][0];
	  func->vname[i] = (char*)varlist->Tuples[i][1];
	}
    }
  
  sprintf (buff, "\n%s\n%s (", func->rtype, func->fname);

  if (varlist->Size () > 0)
    sprintf (buff, "%s %s %s", buff, func->vtype[0], func->vname[0]);
  
  for ( int i = 1; i < varlist->Size ();i++)
    sprintf (buff, "%s,  %s %s",buff, func->vtype[i], func->vname[i]);
  
  sprintf (buff, "%s )\n{\n// put your code below here\n}", buff);
  return true;
} 
